
/* Chrysalide - Outil d'analyse de fichiers binaires
 * project.c - équivalent Python du fichier "analysis/project.c"
 *
 * Copyright (C) 2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "project.h"


#include <assert.h>
#include <pygobject.h>


#include <analysis/project.h>


#include "loaded.h"
#include "../access.h"
#include "../helpers.h"



/* Crée un nouvel objet Python de type 'StudyProject'. */
static PyObject *py_study_project_new(PyTypeObject *, PyObject *, PyObject *);

/* Procède à l'enregistrement d'un projet donné. */
static PyObject *py_study_project_save(PyObject *, PyObject *);

/* Attache un contenu donné à un projet donné. */
static PyObject *py_study_project_attach_content(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : type = type de l'objet à instancier.                         *
*                args = arguments fournis à l'appel.                          *
*                kwds = arguments de type key=val fournis.                    *
*                                                                             *
*  Description : Crée un nouvel objet Python de type 'StudyProject'.          *
*                                                                             *
*  Retour      : Instance Python mise en place.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_study_project_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    PyObject *result;                       /* Instance à retourner        */
    const char *filename;                   /* Destination de la sauvegarde*/
    int ret;                                /* Bilan de lecture des args.  */
    GStudyProject *project;                 /* Version GLib du projet      */

    filename = NULL;

    ret = PyArg_ParseTuple(args, "|s", &filename);
    if (!ret) return NULL;

    if (filename != NULL)
        project = g_study_project_open(filename);
    else
        project = g_study_project_new();

    result = pygobject_new(G_OBJECT(project));

    if (project != NULL)
        g_object_unref(project);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = projet d'étude à manipuler.                           *
*                args = arguments accompagnant l'appel.                       *
*                                                                             *
*  Description : Procède à l'enregistrement d'un projet donné.                *
*                                                                             *
*  Retour      : Py_True si l'enregistrement s'est déroule sans encombre.     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_study_project_save(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Instance à retourner        */
    GStudyProject *project;                 /* Version GLib du format      */
    const char *filename;                   /* Destination de la sauvegarde*/
    int ret;                                /* Bilan de lecture des args.  */
    bool status;                            /* Bilan de l'opération        */

    project = G_STUDY_PROJECT(pygobject_get(self));
    assert(project != NULL);

    ret = PyArg_ParseTuple(args, "s", &filename);
    if (!ret) return NULL;

    status = g_study_project_save(project, filename);

    result = status ? Py_True : Py_False;
    Py_INCREF(result);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = projet d'étude à manipuler.                           *
*                args = arguments accompagnant l'appel.                       *
*                                                                             *
*  Description : Attache un contenu donné à un projet donné.                  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_study_project_attach_content(PyObject *self, PyObject *args)
{
    GStudyProject *project;                 /* Version GLib du format      */
    PyObject *content_obj;                  /* Objet pour le contenu       */
    int ret;                                /* Bilan de lecture des args.  */
    GLoadedContent *content;                /* Instance GLib correspondante*/

    project = G_STUDY_PROJECT(pygobject_get(self));
    assert(project != NULL);

    ret = PyArg_ParseTuple(args, "O!", get_python_loaded_content_type(), &content_obj);
    if (!ret) return NULL;

    content = G_LOADED_CONTENT(pygobject_get(content_obj));

    g_study_project_attach_content(project, content);

    Py_RETURN_NONE;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_study_project_type(void)
{
    static PyMethodDef py_study_project_methods[] = {
        {
            "save", py_study_project_save,
            METH_VARARGS,
            "save($self, filename, /)\n--\n\nSave the project into a given file."
        },
        {
            "attach", py_study_project_attach_content,
            METH_VARARGS,
            "attach($self, loaded, /)\n--\n\nAdd a loaded content to the project."
        },
        { NULL }
    };

    static PyGetSetDef py_study_project_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_study_project_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.StudyProject",

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide study project",

        .tp_methods     = py_study_project_methods,
        .tp_getset      = py_study_project_getseters,
        .tp_new         = py_study_project_new

    };

    return &py_study_project_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.analysis.StudyProject'.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_study_project_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python 'StudyProject'  */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_study_project_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.analysis");

        dict = PyModule_GetDict(module);

        if (!register_class_for_pygobject(dict, G_TYPE_STUDY_PROJECT, type, &PyGObject_Type))
            return false;

    }

    return true;

}
